@@ -6,6 +6,8 @@
 
 """Tests for net_connections() and Process.connections() APIs."""
 
+import contextlib
+import errno
 import os
 import socket
 import textwrap
@@ -29,15 +31,16 @@ from psutil._compat import PY3
 from psutil.tests import AF_UNIX
 from psutil.tests import bind_socket
 from psutil.tests import bind_unix_socket
-from psutil.tests import check_connection_ntuple
+from psutil.tests import check_net_address
 from psutil.tests import create_sockets
+from psutil.tests import enum
 from psutil.tests import get_free_port
 from psutil.tests import HAS_CONNECTIONS_UNIX
 from psutil.tests import pyrun
 from psutil.tests import reap_children
-from psutil.tests import run_test_module_by_name
 from psutil.tests import safe_rmpath
 from psutil.tests import skip_on_access_denied
+from psutil.tests import SKIP_SYSCONS
 from psutil.tests import tcp_socketpair
 from psutil.tests import TESTFN
 from psutil.tests import TRAVIS
@@ -48,6 +51,7 @@ from psutil.tests import wait_for_file
 
 
 thisproc = psutil.Process()
+SOCK_SEQPACKET = getattr(socket, "SOCK_SEQPACKET", object())
 
 
 class Base(object):
@@ -68,6 +72,122 @@ class Base(object):
             cons = thisproc.connections(kind='all')
             assert not cons, cons
 
+    def compare_procsys_connections(self, pid, proc_cons, kind='all'):
+        """Given a process PID and its list of connections compare
+        those against system-wide connections retrieved via
+        psutil.net_connections.
+        """
+        try:
+            sys_cons = psutil.net_connections(kind=kind)
+        except psutil.AccessDenied:
+            # On MACOS, system-wide connections are retrieved by iterating
+            # over all processes
+            if MACOS:
+                return
+            else:
+                raise
+        # Filter for this proc PID and exlucde PIDs from the tuple.
+        sys_cons = [c[:-1] for c in sys_cons if c.pid == pid]
+        sys_cons.sort()
+        proc_cons.sort()
+        self.assertEqual(proc_cons, sys_cons)
+
+    def check_connection_ntuple(self, conn):
+        """Check validity of a connection namedtuple."""
+        def check_ntuple(conn):
+            has_pid = len(conn) == 7
+            self.assertIn(len(conn), (6, 7))
+            self.assertEqual(conn[0], conn.fd)
+            self.assertEqual(conn[1], conn.family)
+            self.assertEqual(conn[2], conn.type)
+            self.assertEqual(conn[3], conn.laddr)
+            self.assertEqual(conn[4], conn.raddr)
+            self.assertEqual(conn[5], conn.status)
+            if has_pid:
+                self.assertEqual(conn[6], conn.pid)
+
+        def check_family(conn):
+            self.assertIn(conn.family, (AF_INET, AF_INET6, AF_UNIX))
+            if enum is not None:
+                assert isinstance(conn.family, enum.IntEnum), conn
+            else:
+                assert isinstance(conn.family, int), conn
+            if conn.family == AF_INET:
+                # actually try to bind the local socket; ignore IPv6
+                # sockets as their address might be represented as
+                # an IPv4-mapped-address (e.g. "::127.0.0.1")
+                # and that's rejected by bind()
+                s = socket.socket(conn.family, conn.type)
+                with contextlib.closing(s):
+                    try:
+                        s.bind((conn.laddr[0], 0))
+                    except socket.error as err:
+                        if err.errno != errno.EADDRNOTAVAIL:
+                            raise
+            elif conn.family == AF_UNIX:
+                self.assertEqual(conn.status, psutil.CONN_NONE)
+
+        def check_type(conn):
+            # SOCK_SEQPACKET may happen in case of AF_UNIX socks
+            self.assertIn(conn.type, (SOCK_STREAM, SOCK_DGRAM, SOCK_SEQPACKET))
+            if enum is not None:
+                assert isinstance(conn.type, enum.IntEnum), conn
+            else:
+                assert isinstance(conn.type, int), conn
+            if conn.type == SOCK_DGRAM:
+                self.assertEqual(conn.status, psutil.CONN_NONE)
+
+        def check_addrs(conn):
+            # check IP address and port sanity
+            for addr in (conn.laddr, conn.raddr):
+                if conn.family in (AF_INET, AF_INET6):
+                    self.assertIsInstance(addr, tuple)
+                    if not addr:
+                        continue
+                    self.assertIsInstance(addr.port, int)
+                    assert 0 <= addr.port <= 65535, addr.port
+                    check_net_address(addr.ip, conn.family)
+                elif conn.family == AF_UNIX:
+                    self.assertIsInstance(addr, str)
+
+        def check_status(conn):
+            self.assertIsInstance(conn.status, str)
+            valids = [getattr(psutil, x) for x in dir(psutil)
+                      if x.startswith('CONN_')]
+            self.assertIn(conn.status, valids)
+            if conn.family in (AF_INET, AF_INET6) and conn.type == SOCK_STREAM:
+                self.assertNotEqual(conn.status, psutil.CONN_NONE)
+            else:
+                self.assertEqual(conn.status, psutil.CONN_NONE)
+
+        check_ntuple(conn)
+        check_family(conn)
+        check_type(conn)
+        check_addrs(conn)
+        check_status(conn)
+
+
+class TestBase(Base, unittest.TestCase):
+
+    @unittest.skipIf(SKIP_SYSCONS, "requires root")
+    def test_system(self):
+        with create_sockets():
+            for conn in psutil.net_connections(kind='all'):
+                self.check_connection_ntuple(conn)
+
+    def test_process(self):
+        with create_sockets():
+            for conn in psutil.Process().connections(kind='all'):
+                self.check_connection_ntuple(conn)
+
+    def test_invalid_kind(self):
+        self.assertRaises(ValueError, thisproc.connections, kind='???')
+        self.assertRaises(ValueError, psutil.net_connections, kind='???')
+
+
+class TestUnconnectedSockets(Base, unittest.TestCase):
+    """Tests sockets which are open but not connected to anything."""
+
     def get_conn_from_sock(self, sock):
         cons = thisproc.connections(kind='all')
         smap = dict([(c.fd, c) for c in cons])
@@ -81,14 +201,13 @@ class Base(object):
                 self.assertEqual(smap[sock.fileno()].fd, sock.fileno())
             return cons[0]
 
-    def check_socket(self, sock, conn=None):
+    def check_socket(self, sock):
         """Given a socket, makes sure it matches the one obtained
         via psutil. It assumes this process created one connection
         only (the one supposed to be checked).
         """
-        if conn is None:
-            conn = self.get_conn_from_sock(sock)
-        check_connection_ntuple(conn)
+        conn = self.get_conn_from_sock(sock)
+        self.check_connection_ntuple(conn)
 
         # fd, family, type
         if conn.fd != -1:
@@ -114,38 +233,9 @@ class Base(object):
         # XXX Solaris can't retrieve system-wide UNIX sockets
         if sock.family == AF_UNIX and HAS_CONNECTIONS_UNIX:
             cons = thisproc.connections(kind='all')
-            self.compare_procsys_connections(os.getpid(), cons)
+            self.compare_procsys_connections(os.getpid(), cons, kind='all')
         return conn
 
-    def compare_procsys_connections(self, pid, proc_cons, kind='all'):
-        """Given a process PID and its list of connections compare
-        those against system-wide connections retrieved via
-        psutil.net_connections.
-        """
-        try:
-            sys_cons = psutil.net_connections(kind=kind)
-        except psutil.AccessDenied:
-            # On MACOS, system-wide connections are retrieved by iterating
-            # over all processes
-            if MACOS:
-                return
-            else:
-                raise
-        # Filter for this proc PID and exlucde PIDs from the tuple.
-        sys_cons = [c[:-1] for c in sys_cons if c.pid == pid]
-        sys_cons.sort()
-        proc_cons.sort()
-        self.assertEqual(proc_cons, sys_cons)
-
-
-# =====================================================================
-# --- Test unconnected sockets
-# =====================================================================
-
-
-class TestUnconnectedSockets(Base, unittest.TestCase):
-    """Tests sockets which are open but not connected to anything."""
-
     def test_tcp_v4(self):
         addr = ("127.0.0.1", get_free_port())
         with closing(bind_socket(AF_INET, SOCK_STREAM, addr=addr)) as sock:
@@ -193,12 +283,7 @@ class TestUnconnectedSockets(Base, unittest.TestCase):
                 self.assertEqual(conn.status, psutil.CONN_NONE)
 
 
-# =====================================================================
-# --- Test connected sockets
-# =====================================================================
-
-
-class TestConnectedSocketPairs(Base, unittest.TestCase):
+class TestConnectedSocket(Base, unittest.TestCase):
     """Test socket pairs which are are actually connected to
     each other.
     """
@@ -258,12 +343,58 @@ class TestConnectedSocketPairs(Base, unittest.TestCase):
                 server.close()
                 client.close()
 
+
+class TestFilters(Base, unittest.TestCase):
+
+    def test_filters(self):
+        def check(kind, families, types):
+            for conn in thisproc.connections(kind=kind):
+                self.assertIn(conn.family, families)
+                self.assertIn(conn.type, types)
+            if not SKIP_SYSCONS:
+                for conn in psutil.net_connections(kind=kind):
+                    self.assertIn(conn.family, families)
+                    self.assertIn(conn.type, types)
+
+        with create_sockets():
+            check('all',
+                  [AF_INET, AF_INET6, AF_UNIX],
+                  [SOCK_STREAM, SOCK_DGRAM, SOCK_SEQPACKET])
+            check('inet',
+                  [AF_INET, AF_INET6],
+                  [SOCK_STREAM, SOCK_DGRAM])
+            check('inet4',
+                  [AF_INET],
+                  [SOCK_STREAM, SOCK_DGRAM])
+            check('tcp',
+                  [AF_INET, AF_INET6],
+                  [SOCK_STREAM])
+            check('tcp4',
+                  [AF_INET],
+                  [SOCK_STREAM])
+            check('tcp6',
+                  [AF_INET6],
+                  [SOCK_STREAM])
+            check('udp',
+                  [AF_INET, AF_INET6],
+                  [SOCK_DGRAM])
+            check('udp4',
+                  [AF_INET],
+                  [SOCK_DGRAM])
+            check('udp6',
+                  [AF_INET6],
+                  [SOCK_DGRAM])
+            if HAS_CONNECTIONS_UNIX:
+                check('unix',
+                      [AF_UNIX],
+                      [SOCK_STREAM, SOCK_DGRAM, SOCK_SEQPACKET])
+
     @skip_on_access_denied(only_if=MACOS)
     def test_combos(self):
         def check_conn(proc, conn, family, type, laddr, raddr, status, kinds):
             all_kinds = ("all", "inet", "inet4", "inet6", "tcp", "tcp4",
                          "tcp6", "udp", "udp4", "udp6")
-            check_connection_ntuple(conn)
+            self.check_connection_ntuple(conn)
             self.assertEqual(conn.family, family)
             self.assertEqual(conn.type, type)
             self.assertEqual(conn.laddr, laddr)
@@ -285,7 +416,7 @@ class TestConnectedSocketPairs(Base, unittest.TestCase):
             import socket, time
             s = socket.socket($family, socket.SOCK_STREAM)
             s.bind(('$addr', 0))
-            s.listen(1)
+            s.listen(5)
             with open('$testfn', 'w') as f:
                 f.write(str(s.getsockname()[:2]))
             time.sleep(60)
@@ -353,13 +484,8 @@ class TestConnectedSocketPairs(Base, unittest.TestCase):
                                psutil.CONN_NONE,
                                ("all", "inet", "inet6", "udp", "udp6"))
 
-        # err
-        self.assertRaises(ValueError, p.connections, kind='???')
-
-    def test_multi_sockets_filtering(self):
-        with create_sockets() as socks:
-            cons = thisproc.connections(kind='all')
-            self.assertEqual(len(cons), len(socks))
+    def test_count(self):
+        with create_sockets():
             # tcp
             cons = thisproc.connections(kind='tcp')
             self.assertEqual(len(cons), 2 if supports_ipv6() else 1)
@@ -407,8 +533,9 @@ class TestConnectedSocketPairs(Base, unittest.TestCase):
                 for conn in cons:
                     self.assertEqual(conn.family, AF_INET6)
                     self.assertIn(conn.type, (SOCK_STREAM, SOCK_DGRAM))
-            # unix
-            if HAS_CONNECTIONS_UNIX:
+            # unix (skipped on NetBSD because by default the Python process
+            # creates a connection to '/var/run/log' UNIX socket)
+            if HAS_CONNECTIONS_UNIX and not NETBSD:
                 cons = thisproc.connections(kind='unix')
                 self.assertEqual(len(cons), 3)
                 for conn in cons:
@@ -416,23 +543,17 @@ class TestConnectedSocketPairs(Base, unittest.TestCase):
                     self.assertIn(conn.type, (SOCK_STREAM, SOCK_DGRAM))
 
 
-# =====================================================================
-# --- Miscellaneous tests
-# =====================================================================
-
-
+@unittest.skipIf(SKIP_SYSCONS, "requires root")
 class TestSystemWideConnections(Base, unittest.TestCase):
     """Tests for net_connections()."""
 
-    @skip_on_access_denied()
     def test_it(self):
         def check(cons, families, types_):
-            AF_UNIX = getattr(socket, 'AF_UNIX', object())
             for conn in cons:
                 self.assertIn(conn.family, families, msg=conn)
                 if conn.family != AF_UNIX:
                     self.assertIn(conn.type, types_, msg=conn)
-                check_connection_ntuple(conn)
+                self.check_connection_ntuple(conn)
 
         with create_sockets():
             from psutil._common import conn_tmap
@@ -445,16 +566,6 @@ class TestSystemWideConnections(Base, unittest.TestCase):
                 self.assertEqual(len(cons), len(set(cons)))
                 check(cons, families, types_)
 
-            self.assertRaises(ValueError, psutil.net_connections, kind='???')
-
-    @skip_on_access_denied()
-    def test_multi_socks(self):
-        with create_sockets() as socks:
-            cons = [x for x in psutil.net_connections(kind='all')
-                    if x.pid == os.getpid()]
-            self.assertEqual(len(cons), len(socks))
-
-    @skip_on_access_denied()
     # See: https://travis-ci.org/giampaolo/psutil/jobs/237566297
     @unittest.skipIf(MACOS and TRAVIS, "unreliable on MACOS + TRAVIS")
     def test_multi_sockets_procs(self):
@@ -496,11 +607,6 @@ class TestSystemWideConnections(Base, unittest.TestCase):
             self.assertEqual(len(p.connections('all')), expected)
 
 
-# =====================================================================
-# --- Miscellaneous tests
-# =====================================================================
-
-
 class TestMisc(unittest.TestCase):
 
     def test_connection_constants(self):
@@ -523,4 +629,5 @@ class TestMisc(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    run_test_module_by_name(__file__)
+    from psutil.tests.runner import run
+    run(__file__)
