@@ -14,7 +14,6 @@ for some reason).
 """
 
 from __future__ import print_function
-import errno
 import functools
 import gc
 import os
@@ -30,14 +29,18 @@ from psutil import OPENBSD
 from psutil import POSIX
 from psutil import SUNOS
 from psutil import WINDOWS
+from psutil._common import bytes2human
+from psutil._compat import ProcessLookupError
 from psutil._compat import xrange
 from psutil.tests import create_sockets
 from psutil.tests import get_test_subprocess
 from psutil.tests import HAS_CPU_AFFINITY
 from psutil.tests import HAS_CPU_FREQ
+from psutil.tests import HAS_GETLOADAVG
 from psutil.tests import HAS_ENVIRON
 from psutil.tests import HAS_IONICE
 from psutil.tests import HAS_MEMORY_MAPS
+from psutil.tests import HAS_NET_IO_COUNTERS
 from psutil.tests import HAS_PROC_CPU_NUM
 from psutil.tests import HAS_PROC_IO_COUNTERS
 from psutil.tests import HAS_RLIMIT
@@ -45,7 +48,6 @@ from psutil.tests import HAS_SENSORS_BATTERY
 from psutil.tests import HAS_SENSORS_FANS
 from psutil.tests import HAS_SENSORS_TEMPERATURES
 from psutil.tests import reap_children
-from psutil.tests import run_test_module_by_name
 from psutil.tests import safe_rmpath
 from psutil.tests import skip_on_access_denied
 from psutil.tests import TESTFN
@@ -73,25 +75,6 @@ def skip_if_linux():
                            "worthless on LINUX (pure python)")
 
 
-def bytes2human(n):
-    """
-    http://code.activestate.com/recipes/578019
-    >>> bytes2human(10000)
-    '9.8K'
-    >>> bytes2human(100001221)
-    '95.4M'
-    """
-    symbols = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
-    prefix = {}
-    for i, s in enumerate(symbols):
-        prefix[s] = 1 << (i + 1) * 10
-    for s in reversed(symbols):
-        if n >= prefix[s]:
-            value = float(n) / prefix[s]
-            return '%.2f%s' % (value, s)
-    return "%sB" % n
-
-
 class TestMemLeak(unittest.TestCase):
     """Base framework class which calls a function many times and
     produces a failure if process memory usage keeps increasing
@@ -440,9 +423,8 @@ class TestTerminatedProcessLeaks(TestProcessObjectLeaks):
             def call():
                 try:
                     return cext.proc_info(self.proc.pid)
-                except OSError as err:
-                    if err.errno != errno.ESRCH:
-                        raise
+                except ProcessLookupError:
+                    pass
 
             self.execute(call)
 
@@ -484,6 +466,7 @@ class TestModuleFunctionsLeaks(TestMemLeak):
     def test_per_cpu_times(self):
         self.execute(psutil.cpu_times, percpu=True)
 
+    @skip_if_linux()
     def test_cpu_stats(self):
         self.execute(psutil.cpu_stats)
 
@@ -492,6 +475,10 @@ class TestModuleFunctionsLeaks(TestMemLeak):
     def test_cpu_freq(self):
         self.execute(psutil.cpu_freq)
 
+    @unittest.skipIf(not HAS_GETLOADAVG, "not supported")
+    def test_getloadavg(self):
+        self.execute(psutil.getloadavg)
+
     # --- mem
 
     def test_virtual_memory(self):
@@ -534,6 +521,7 @@ class TestModuleFunctionsLeaks(TestMemLeak):
 
     @unittest.skipIf(TRAVIS and MACOS, "false positive on travis")
     @skip_if_linux()
+    @unittest.skipIf(not HAS_NET_IO_COUNTERS, 'not supported')
     def test_net_io_counters(self):
         self.execute(psutil.net_io_counters, nowrap=False)
 
@@ -605,4 +593,5 @@ class TestModuleFunctionsLeaks(TestMemLeak):
 
 
 if __name__ == '__main__':
-    run_test_module_by_name(__file__)
+    from psutil.tests.runner import run
+    run(__file__)
